clear all;
clc;
close all;

%% UCA settings
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;

%% distance estimation with Bessel Function J0
theta_0 = 0;
r_0 = 20; % the first focal point
r_1 = 30; % the second focal point

nt_list = 2^5:2^2:2^12+2^10; % change the number of antennas
r_radius_list = nt_list*d/2/pi; % calculate the radius of UCA
num_nt = length(nt_list);
g_r = zeros(1, num_nt);
g_r_est = zeros(1, num_nt);
g_bound= zeros(1, num_nt);
for idx_nt = 1:num_nt
    Nt = nt_list(idx_nt);
    r_radius = Nt*d/2/pi;
    [at,~] = generate_UCA(fc, Nt, r_radius, r_0, theta_0);
    [h0,~] = generate_UCA(fc, Nt, r_radius, r_1, theta_0);
    g_r(idx_nt)=  abs(at*h0');
    
    xi = -2*pi*r_radius^2/lambda/4*(1/(r_1)-1/r_0);
    g_r_est(idx_nt) = abs(besselj(0, xi));
    
    g_bound(idx_nt) = 2/pi/r_radius*sqrt(lambda/(1/r_0-1/r_1));
end

figure;
hold on;
plot(r_radius_list, g_r,'r-','LineWidth',1.5);
plot(r_radius_list, g_r_est,'b--','LineWidth',1.5);
plot(r_radius_list, g_bound,'k--','LineWidth',1.5);
grid on;
box on;
xlabel('Radius (m)');
ylabel('Beamforming Gain');
legend('Calculated','Estimated with (11)','Upper bound in (15)','Location','NorthEast');
axis([r_radius_list(1) r_radius_list(end) 0  1]);