clear all;
clc;
close all;

%% Cylindrical array settings
M = 51;
Nt = 600;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;

r_radius = Nt*d/2/pi; % half-wavelength spacing
D_frau = 2*(2*r_radius)^2/lambda;
D_fres = (2*r_radius)^(4/3)/2/(lambda)^(1/3);


%% plot the beam gain over distance
theta_0 = pi/2;
r_0 = 100000;
phi_0 = 0;
at = generate_cylindrical(fc, Nt, M, r_radius, d, r_0, theta_0, phi_0);

h_0 = cylindrical_far_beam(fc, Nt, M, r_radius, d, theta_0, phi_0);
g = abs(h_0*at');

num_r = 4000;
r_list = linspace(D_fres, 30, num_r);
g_r = zeros(1, num_r);
g_r_app = zeros(1, num_r);
g_r_app2 = zeros(1, num_r);
g_r_est = zeros(1, num_r);
g_r_est2 = zeros(1, num_r);
num_point = 100000;
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    h0 = generate_cylindrical(fc, Nt, M, r_radius, d, r_iter, theta_0, phi_0);
    g_r(idx_r)=  abs(at*h0');
    
    [at_near, at_near_app] = cylindrical_near_beam(fc, Nt, M, r_radius, d, r_iter, theta_0, phi_0);
    g_r_app(idx_r) = abs(at*at_near');
    g_r_app2(idx_r) = abs(at*at_near_app');
    
    beta = sqrt(M^2*d^2/2/lambda*abs(1/r_0-1/r_iter));
    [f_sin, f_cos] = fresnel(beta, num_point);
    G = abs(f_cos+1i*f_sin)/beta;
    
    xi = pi*r_radius^2/2/lambda*(1/r_0-1/r_iter);
    g_r_est(idx_r) = abs(besselj(0, xi)*G);
end
plot_line = 0:0.01:1;
figure;
hold on;
plot(r_list, g_r,'r-','LineWidth',1.5);
% plot(r_list, g_r_app,'k-','LineWidth',1.5);
plot(r_list, g_r_est,'b--','LineWidth',1.5);
plot(D_fres*ones(1, length(plot_line)), plot_line,'r--','LineWidth',1.5);
% plot(D_frau*ones(1, length(plot_line)), plot_line,'r--','LineWidth',1.5);
grid on;
box on;
xlabel('Distance');
ylabel('Beamforming Gain');
title('Cylindrical array');
legend('Accurate','Estimated');

%% change with M
theta_0 = pi/2;
r_0 = 100000;
phi_0 = 0;

num_r = 4000;
r_list = linspace(D_fres, 30, num_r);
M_list = 1:128:513;
num_m = length(M_list);
g_r = zeros(num_m, num_r);
g_r_est = zeros(num_m, num_r);

for i_m = 1:num_m
    M = M_list(i_m);
    at = generate_cylindrical(fc, Nt, M, r_radius, d, r_0, theta_0, phi_0);
    for idx_r = 1:num_r
        r_iter = r_list(idx_r);
        h0 = generate_cylindrical(fc, Nt, M, r_radius, d, r_iter, theta_0, phi_0);
        g_r(i_m, idx_r)=  abs(at*h0');
        
        beta = sqrt(M^2*d^2/2/lambda*abs(1/r_0-1/r_iter));
        [f_sin, f_cos] = fresnel(beta, num_point);
        G = abs(f_cos+1i*f_sin)/beta;
        xi = pi*r_radius^2/2/lambda*(1/r_0-1/r_iter);
        g_r_est(i_m, idx_r) = abs(besselj(0, xi)*G);
    end
end
figure;
hold on;
plot(r_list, g_r,'LineWidth',1.5);
plot(r_list, g_r_est,'k--','LineWidth',1.5);
grid on;
box on;
xlabel('Distance (m)');
ylabel('Beamforming Gain');
title('Cylindrical array');
legend('M=1','M=129','M=257','M=385','M=513', 'Estimated with (25)');

