clear all;
clc;
close all;

%% UCA settings
Nt = 800;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;

r_radius = Nt*d/2/pi; % half-wavelength spacing
D = 2*(2*r_radius)^2/lambda;

%% estimation of near-field angular beamforming
dis_list = [5,10,100,1000];
num_dis = length(dis_list);
theta_0 = 0;

num_theta = 10000;
theta_list = linspace(-pi/20, pi/20, num_theta);
g = zeros(num_dis, num_theta);
g_est = zeros(num_dis, num_theta);
for idx_dis = 1:num_dis
    dis_iter = dis_list(idx_dis);
    at = generate_UCA(fc, Nt, r_radius, dis_iter, theta_0);
    for idx_theta = 1:num_theta
        theta_iter = theta_list(idx_theta);
        h0 = generate_UCA(fc, Nt, r_radius, dis_iter, theta_iter);
        g(idx_dis,idx_theta)=  abs(at*h0');
        g_est(idx_dis,idx_theta) = abs(besselj(0, -2*pi*r_radius/lambda*2*sin((theta_iter-theta_0)/2)));
    end
end
figure;
hold on;
plot(theta_list, g(1,:),'c-','LineWidth',1.5);
plot(theta_list, g(2,:),'g-','LineWidth',1.5);
plot(theta_list, g(3,:),'m-','LineWidth',1.5);
plot(theta_list, g(4,:),'r-','LineWidth',1.5);
plot(theta_list, g_est(1,:),'b--','LineWidth',1.5);
err = g-g_est;
grid on;
box on;
xlabel('\phi_2 - \phi_1','FontSize',13);
ylabel('Beamforming Gain');
legend('r = 5 m','r = 10 m','r = 100 m','r = 1000 m','Estimated with (9)');
axis([theta_list(1) theta_list(end) 0 1]);

%% distance estimation with Bessel Function J0
theta_0 = 0;
r_0 = 20;
at = generate_UCA(fc, Nt, r_radius, r_0, theta_0);

pp = 1:1:Nt;
phi_p = (pp-1)*2*pi/Nt;
[x_p, y_p] = pol2cart(phi_p, r_radius);
[x_0, y_0] = pol2cart(theta_0, r_0);

num_r = 10000;
r_list = linspace(3, 50, num_r);
g_r = zeros(1, num_r);
g_r_est = zeros(1, num_r);
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    h0 = generate_UCA(fc, Nt, r_radius, r_iter, theta_0);
    g_r(idx_r)=  abs(at*h0');

    xi = -2*pi*r_radius^2/lambda/4*(1/(r_iter)-1/r_0);
    g_r_est(idx_r) = abs(besselj(0, xi));
end
xi_3db = 1.522;
r_left = pi*r_radius^2*r_0/(pi*r_radius^2+2*xi_3db*lambda*r_0);
r_right = pi*r_radius^2*r_0/(pi*r_radius^2-2*xi_3db*lambda*r_0);
if r_right < 0
    r_right = r_left;
end
line_plot = 0:0.01:1;
num_plot = length(line_plot);


figure;
hold on;
plot(r_list, g_r,'r-','LineWidth',1.5);
plot(r_list, g_r_est,'b--','LineWidth',1.5);
plot(r_left*ones(1,num_plot), line_plot, 'k--','LineWidth',1);
plot(r_right*ones(1,num_plot), line_plot, 'k--','LineWidth',1);
grid on;
box on;
xlabel('Distance (m)');
ylabel('Beamforming Gain');
legend('Calculated','Estimated with (8)','Edge of 3 dB range','Location','NorthEast');




