clear all;
clc;
close all;

%% parameters
Nt_ULA = 256;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;

% find corresponding point for threshold
threshold = 0.5;
num_bessel = 1000;
r_list = linspace(0, 4, num_bessel);
g_r = abs(besselj(0, r_list));
[~,idx] = min(abs(g_r-threshold));
xi_delta = r_list(idx);

% xi_delta = 1.5205;
xi_delta_zero = 2.4048;

r_radius = Nt_ULA*d/2; % half-wavelength spacing
D_frau = 2*(2*r_radius)^2/lambda;
D_fres = (2*r_radius)^(4/3)/2/(lambda)^(1/3);
Nt = 256;
Nt_uca = 2*pi*r_radius/d;

delta_phi = asin(xi_delta*lambda/4/pi/r_radius)*2;
num_phi_beam = 2*pi/delta_phi;

delta_phi_zero = asin(xi_delta_zero*lambda/4/pi/r_radius)*2;
num_phi_zero = 2*pi/delta_phi_zero;

%% verify the codebook
num_beam_phi = floor(num_phi_beam);
phi_list_tmp = (0:num_beam_phi-1)*delta_phi;
beam_phi_list = UCA_far_beam(fc, Nt, r_radius, phi_list_tmp);
correlation_phi = beam_phi_list*beam_phi_list';
correlation_phi = abs(correlation_phi-diag(diag(correlation_phi)));
max_corr_phi = max(correlation_phi);

%% plot the multi-beam in azimuth domain
r_near = 10;
r_far = 10000;
num_phi_plot = 10000;
plot_idx = 10;
phi_list_plot = linspace(0, pi/160, num_phi_plot);
gain_phi = zeros(plot_idx, num_phi_plot);
for idx_phi_plot = 1:num_phi_plot
    phi_iter = phi_list_plot(idx_phi_plot);
    h0 = generate_UCA(fc, Nt, r_radius, r_far, phi_iter);
    gain_phi(:, idx_phi_plot)=  abs(beam_phi_list(1:plot_idx,:)*h0');
end
figure;
plot(phi_list_plot, gain_phi,'LineWidth',1.5);
xlabel('\phi', 'FontSize', 13);
ylabel('Beamforming Gain');
grid on;
box on;


%% codebook in the distance domain
r_min = 6;
delta_distance = xi_delta*2*lambda/r_radius^2/pi;
num_distance_beam = 1/r_min/delta_distance;

delta_distance_zero = xi_delta_zero*2*lambda/r_radius^2/pi;
num_distance_zero = 1/r_min/delta_distance_zero;

%% verify the codebook
num_beam_distance = floor(num_distance_beam);
dis_list_tmp = zeros(1, num_beam_distance);
for i_dis = 1:num_beam_distance
    if i_dis == 1
        dis_list_tmp(i_dis) = 10000;
    else
        dis_list_tmp(i_dis) = 1/delta_distance/(i_dis-1);
    end
end
phi_0 = 0;
phi_list_for_dis = phi_0*ones(1, num_beam_distance);
beam_dis_list = generate_UCA(fc, Nt, r_radius, dis_list_tmp, phi_list_for_dis);
correlation_dis = beam_dis_list*beam_dis_list';
correlation_dis = abs(correlation_dis-diag(diag(correlation_dis)));
max_corr_dis = max(correlation_dis);

% plot the multi-beam in the distance domain
num_r = 10000;
r_list = linspace(r_min, 100, num_r);
gain_beams = zeros(num_beam_distance, num_r);
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    h0 = generate_UCA(fc, Nt, r_radius, r_iter, phi_0);
    gain_beams(:, idx_r) = abs(beam_dis_list*h0');
end
figure;
plot(r_list, gain_beams,'LineWidth',1.5);
xlabel('Distance (m)');
ylabel('Beamforming Gain');
grid on;
box on;

%% polar domain codebook
num_code = num_beam_distance*num_beam_phi;
polar_codebook = zeros(num_code, Nt);
cnt = 1;
for i_dis = 1:num_beam_distance
    r_value = dis_list_tmp(i_dis);
    for j_phi = 1:num_beam_phi
        phi_value = phi_list_tmp(j_phi);
        polar_codebook(cnt, :) = generate_UCA(fc, Nt, r_radius, r_value, phi_value);
        cnt = cnt + 1;
    end
end
corr_codebook = polar_codebook*polar_codebook';
corr_codebook = abs(corr_codebook-diag(diag(corr_codebook)));
max_codebook = (max(corr_codebook));

